'use client';
"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.getPatternGroups = getPatternGroups;
exports.getInputSelectedState = getInputSelectedState;
exports.validateDateTime = validateDateTime;
exports.modifyDate = modifyDate;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _dateUtils = require("../utils/dateUtils");
function getPatternGroups(format, pattern) {
  var _format$match;
  return ((_format$match = format.match(new RegExp("(" + pattern + ")+"))) === null || _format$match === void 0 ? void 0 : _format$match[0]) || '';
}
function getInputSelectedState(options) {
  var input = options.input,
    direction = options.direction,
    formatStr = options.formatStr,
    localize = options.localize,
    selectedMonth = options.selectedMonth,
    dateString = options.dateString,
    _options$valueOffset = options.valueOffset,
    valueOffset = _options$valueOffset === void 0 ? 0 : _options$valueOffset;
  var getSelectIndexGap = function getSelectIndexGap(pattern) {
    var gap = 0;
    var monthIsAbbreviated = formatStr.includes('MMM');
    var monthIsFull = formatStr.includes('MMMM');

    // If the month is abbreviated or full, the gap needs to be adjusted.
    if (monthIsAbbreviated || monthIsFull) {
      var isSelected = pattern === 'M';
      if (selectedMonth === null && valueOffset === 0) {
        return gap;
      }
      var month = selectedMonth ? selectedMonth + (isSelected ? valueOffset : 0) : 1;
      if (month > 12) {
        month = 1;
      } else if (month === 0) {
        month = 12;
      }
      var monthStr = localize === null || localize === void 0 ? void 0 : localize.month(month - 1, {
        width: monthIsFull ? 'wide' : 'abbreviated'
      });
      gap = monthStr.length - (monthIsFull ? 4 : 3);
    }
    return gap;
  };
  var getDatePattern = function getDatePattern(selectionIndex, positionOffset) {
    if (positionOffset === void 0) {
      positionOffset = -1;
    }
    var pattern = formatStr.charAt((input === null || input === void 0 ? void 0 : input.selectionStart) || 0);
    if (selectionIndex < 0 || selectionIndex > dateString.length - 1) {
      pattern = formatStr.trim().charAt(0);
      return pattern;
    }
    var gap = getSelectIndexGap(pattern);
    pattern = formatStr.charAt(selectionIndex - gap);

    // If the pattern is not a letter, then get the pattern from the previous or next letter.
    if (!pattern.match(/[y|d|M|H|h|m|s|a]/)) {
      var nextIndex = selectionIndex + positionOffset;
      pattern = getDatePattern(nextIndex, positionOffset);
    }
    return pattern;
  };
  var getPatternSelectedIndexes = function getPatternSelectedIndexes(pattern) {
    var selectionStart = formatStr.indexOf(pattern);
    var selectionEnd = formatStr.lastIndexOf(pattern) + 1;
    var gap = getSelectIndexGap(pattern);
    if (pattern === 'M') {
      return {
        selectionStart: selectionStart,
        selectionEnd: selectionEnd + gap
      };
    }
    return {
      selectionStart: selectionStart + gap,
      selectionEnd: selectionEnd + gap
    };
  };
  if (typeof input.selectionEnd === 'number' && typeof input.selectionStart === 'number') {
    var index = input.selectionStart;
    var positionOffset = -1;
    if (direction === 'left') {
      index = input.selectionStart - 1;
    } else if (direction === 'right') {
      index = input.selectionEnd + 1;
      positionOffset = 1;
    }
    var datePattern = getDatePattern(index, positionOffset);
    var indexes = getPatternSelectedIndexes(datePattern);
    return (0, _extends2.default)({
      selectedPattern: datePattern
    }, indexes);
  }
  return {
    selectedPattern: 'y',
    selectionStart: 0,
    selectionEnd: 0
  };
}
function validateDateTime(type, value) {
  switch (type) {
    case 'year':
      if (value < 1 || value > 9999) {
        return false;
      }
      break;
    case 'month':
      if (value < 1 || value > 12) {
        return false;
      }
      break;
    case 'day':
      if (value < 1 || value > 31) {
        return false;
      }
      break;
    case 'hour':
      if (value < 0 || value > 23) {
        return false;
      }
      break;
    case 'minute':
      if (value < 0 || value > 59) {
        return false;
      }
      break;
    case 'second':
      if (value < 0 || value > 59) {
        return false;
      }
      break;
    default:
      return false;
    // Invalid type
  }

  return true;
}
function modifyDate(date, type, value) {
  switch (type) {
    case 'year':
      return (0, _dateUtils.setYear)(date, value);
    case 'month':
      return (0, _dateUtils.setMonth)(date, value - 1);
    case 'day':
      return (0, _dateUtils.setDate)(date, value);
    case 'hour':
      return (0, _dateUtils.setHours)(date, value);
    case 'minute':
      return (0, _dateUtils.setMinutes)(date, value);
    case 'second':
      return (0, _dateUtils.setSeconds)(date, value);
  }
  return date;
}