'use client';
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React, { useState, useCallback, useRef, useMemo } from 'react';
import Input from '../Input';
import { mergeRefs, useCustom, useControlled, safeSetSelection } from '../utils';
import { getInputSelectedState, validateDateTime, getPatternGroups } from './utils';
import useDateInputState from './useDateInputState';
var isTestEnvironment = undefined === 'test';
/**
 * The DateInput component lets users select a date with the keyboard.
 * @version 5.58.0
 * @see https://rsuitejs.com/components/date-input/
 */
var DateInput = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var _props$format = props.format,
    formatStr = _props$format === void 0 ? 'yyyy-MM-dd' : _props$format,
    valueProp = props.value,
    defaultValue = props.defaultValue,
    onChange = props.onChange,
    onKeyDown = props.onKeyDown,
    rest = _objectWithoutPropertiesLoose(props, ["format", "value", "defaultValue", "onChange", "onKeyDown"]);
  var inputRef = useRef();
  var _useState = useState({
      selectedPattern: 'y',
      selectionStart: 0,
      selectionEnd: 0
    }),
    selectedState = _useState[0],
    setSelectedState = _useState[1];
  var _useCustom = useCustom('Calendar'),
    locale = _useCustom.locale;
  var localize = locale.dateLocale.localize;
  var _useControlled = useControlled(valueProp, defaultValue),
    value = _useControlled[0],
    setValue = _useControlled[1],
    isControlled = _useControlled[2];
  var _useDateInputState = useDateInputState({
      formatStr: formatStr,
      localize: localize,
      date: value,
      isControlledDate: isControlled
    }),
    dateField = _useDateInputState.dateField,
    setDateOffset = _useDateInputState.setDateOffset,
    setDateField = _useDateInputState.setDateField,
    getDateField = _useDateInputState.getDateField,
    toDateString = _useDateInputState.toDateString;
  var dateString = toDateString();
  var keyPressOptions = useMemo(function () {
    return {
      formatStr: formatStr,
      localize: localize,
      selectedMonth: dateField.month,
      dateString: dateString
    };
  }, [dateField, dateString, formatStr, localize]);
  var setSelectionRange = useCallback(function (selectionStart, selectionEnd) {
    if (selectionStart === void 0) {
      selectionStart = selectedState.selectionStart;
    }
    if (selectionEnd === void 0) {
      selectionEnd = selectedState.selectionEnd;
    }
    var input = inputRef === null || inputRef === void 0 ? void 0 : inputRef.current;
    if (isTestEnvironment) {
      safeSetSelection(input, selectionStart, selectionEnd);
      return;
    }
    requestAnimationFrame(function () {
      safeSetSelection(input, selectionStart, selectionEnd);
    });
  }, [selectedState]);
  var handleChange = useCallback(function (value, event) {
    setValue(value);
    onChange === null || onChange === void 0 ? void 0 : onChange(value, event);
  }, [onChange, setValue]);
  var handleChangeField = useCallback(function (event, nextDirection) {
    var input = event.target;
    var direction = nextDirection || (event.key === 'ArrowRight' ? 'right' : 'left');
    var state = getInputSelectedState(_extends({}, keyPressOptions, {
      input: input,
      direction: direction
    }));
    setSelectionRange(state.selectionStart, state.selectionEnd);
    setSelectedState(state);
  }, [keyPressOptions, setSelectionRange]);
  var handleChangeFieldValue = useCallback(function (event) {
    var key = event.key;
    var input = event.target;
    var offset = key === 'ArrowUp' ? 1 : -1;
    var state = getInputSelectedState(_extends({}, keyPressOptions, {
      input: input,
      valueOffset: offset
    }));
    setSelectedState(state);
    setDateOffset(state.selectedPattern, offset, function (date) {
      return handleChange(date, event);
    });
    setSelectionRange(state.selectionStart, state.selectionEnd);
  }, [handleChange, keyPressOptions, setDateOffset, setSelectionRange]);
  var isFieldFullValue = useCallback(function (value, pattern) {
    var patternGroup = getPatternGroups(formatStr, pattern);
    if (value.toString().length === patternGroup.length) {
      return true;
    }
    switch (pattern) {
      case 'M':
        return parseInt(value + "0") > 12;
      case 'd':
        return parseInt(value + "0") > 31;
      case 'H':
        return parseInt(value + "0") > 23;
      case 'h':
        return parseInt(value + "0") > 12;
      case 'm':
      case 's':
        return parseInt(value + "0") > 59;
      default:
        return false;
    }
  }, [formatStr]);
  var handleChangeFieldValueWithNumericKeys = useCallback(function (event) {
    var key = event.key;
    var input = event.target;
    var pattern = selectedState.selectedPattern;
    if (!pattern) {
      return;
    }
    var field = getDateField(pattern);
    var value = parseInt(key, 10);
    var padValue = parseInt("" + (field.value || '') + key, 10);
    var newValue = value;

    // Check if the value entered by the user is a valid date
    if (validateDateTime(field.name, padValue)) {
      newValue = padValue;
    }
    if (pattern === 'M') {
      // Month cannot be less than 1.
      newValue = Math.max(1, newValue);
    }
    setDateField(pattern, newValue, function (date) {
      return handleChange(date, event);
    });

    // The currently selected month will be retained as a parameter of getInputSelectedState,
    // but if the user enters a month, the month value will be replaced with the value entered by the user.
    var selectedMonth = pattern === 'M' ? newValue : dateField.month;
    var nextState = getInputSelectedState(_extends({}, keyPressOptions, {
      input: input,
      selectedMonth: selectedMonth
    }));
    setSelectedState(nextState);
    setSelectionRange(nextState.selectionStart, nextState.selectionEnd);

    // If the field is full value, move the cursor to the next field
    if (isFieldFullValue(newValue, pattern) && input.selectionEnd !== input.value.length) {
      handleChangeField(event, 'right');
    }
  }, [dateField, getDateField, handleChange, handleChangeField, isFieldFullValue, keyPressOptions, selectedState.selectedPattern, setDateField, setSelectionRange]);
  var handleRemoveFieldValue = useCallback(function (event) {
    if (selectedState.selectedPattern) {
      setDateField(selectedState.selectedPattern, null, function (date) {
        return handleChange(date, event);
      });
      setSelectionRange();
    }
  }, [handleChange, selectedState, setDateField, setSelectionRange]);
  var handleKeyDown = function handleKeyDown(event) {
    var _key$match;
    var key = event.key;
    switch (key) {
      case 'ArrowRight':
      case 'ArrowLeft':
        handleChangeField(event);
        event.preventDefault();
        break;
      case 'ArrowUp':
      case 'ArrowDown':
        handleChangeFieldValue(event);
        event.preventDefault();
        break;
      case 'Backspace':
        handleRemoveFieldValue(event);
        event.preventDefault();
        break;
      case (_key$match = key.match(/\d/)) === null || _key$match === void 0 ? void 0 : _key$match.input:
        handleChangeFieldValueWithNumericKeys(event);
        event.preventDefault();
        break;
    }
    onKeyDown === null || onKeyDown === void 0 ? void 0 : onKeyDown(event);
  };
  var handleClick = useCallback(function (event) {
    var input = event.target;
    var state = getInputSelectedState(_extends({}, keyPressOptions, {
      input: input
    }));
    setSelectedState(state);
    setSelectionRange(state.selectionStart, state.selectionEnd);
  }, [keyPressOptions, setSelectionRange]);
  return /*#__PURE__*/React.createElement(Input, _extends({
    inputMode: "text",
    autoComplete: "off",
    autoCorrect: "off",
    spellCheck: false,
    ref: mergeRefs(inputRef, ref),
    onKeyDown: handleKeyDown,
    onClick: handleClick,
    value: dateString
  }, rest));
});
DateInput.displayName = 'DateInput';
export default DateInput;