'use client';
import { useCallback } from 'react';
import startCase from 'lodash/startCase';
import { addDays, addMonths, addYears, addHours, addMinutes, addSeconds, format, isLastDayOfMonth, lastDayOfMonth } from '../utils/dateUtils';
import { useDateField, patternMap } from './DateField';
function useDateInputState(_ref) {
  var formatStr = _ref.formatStr,
    localize = _ref.localize,
    date = _ref.date,
    isControlledDate = _ref.isControlledDate;
  var _useDateField = useDateField(formatStr, localize, date),
    dateField = _useDateField.dateField,
    dispatch = _useDateField.dispatch,
    toDateString = _useDateField.toDateString,
    toDate = _useDateField.toDate;
  var setDateOffset = useCallback(function (pattern, offset, callback) {
    var currentDate = new Date();
    var year = dateField.year || currentDate.getFullYear();
    var month = dateField.month ? dateField.month - 1 : currentDate.getMonth();
    var day = dateField.day || 0;
    var hour = dateField.hour || 0;
    var minute = dateField.minute || 0;
    var second = dateField.second || 0;
    var actionName;
    var value;
    switch (pattern) {
      case 'y':
        actionName = 'setYear';
        value = addYears(new Date(year, 0), offset).getFullYear();
        break;
      case 'M':
        actionName = 'setMonth';
        value = addMonths(new Date(year, month), offset).getMonth() + 1;
        break;
      case 'd':
        actionName = 'setDay';
        var prevDate = new Date(year, month, day);
        value = addDays(prevDate, offset).getDate();
        if (offset > 0) {
          value = isLastDayOfMonth(prevDate) ? 1 : value;
        } else {
          value = prevDate.getDate() === 1 ? lastDayOfMonth(prevDate).getDate() : value;
        }
        break;
      case 'H':
      case 'h':
        actionName = 'setHour';
        value = addHours(new Date(year, month, day, hour), offset).getHours();
        break;
      case 'm':
        actionName = 'setMinute';
        value = addMinutes(new Date(year, month, day, hour, minute), offset).getMinutes();
        break;
      case 's':
        actionName = 'setSecond';
        value = addSeconds(new Date(year, month, day, hour, minute, second), offset).getSeconds();
        break;
      case 'a':
        actionName = 'setMeridian';
        value = dateField.meridian === 'AM' ? 'PM' : 'AM';
        break;
    }
    if (actionName && value) {
      dispatch({
        type: actionName,
        value: value
      });
      var field = patternMap[pattern];
      callback === null || callback === void 0 ? void 0 : callback(toDate(field, value));
    }
  }, [dateField, toDate, dispatch]);
  var setDateField = useCallback(function (pattern, value, callback) {
    var field = patternMap[pattern];
    var actionName = "set" + startCase(field);
    dispatch({
      type: actionName,
      value: value
    });
    callback === null || callback === void 0 ? void 0 : callback(toDate(field, value));
  }, [toDate, dispatch]);
  var getDateField = useCallback(function (pattern) {
    var fieldName = patternMap[pattern];
    return {
      name: fieldName,
      value: dateField[fieldName]
    };
  }, [dateField]);
  var toControlledDateString = useCallback(function () {
    return format(date || new Date(), formatStr);
  }, [date, formatStr]);
  return {
    dateField: dateField,
    setDateOffset: setDateOffset,
    setDateField: setDateField,
    getDateField: getDateField,
    toDateString: isControlledDate ? toControlledDateString : toDateString
  };
}
export default useDateInputState;